@@ -1,15 +1,15 @@
 import os
-from busco.BuscoConfig import BuscoConfigAuto
 from busco.BuscoPlacer import BuscoPlacer
 from busco.BuscoLogger import BuscoLogger
 from busco.BuscoLogger import LogDecorator as log
-from busco.BuscoRunner import BuscoRunner
+from busco.BuscoRunner import AnalysisRunner
+from busco.Exceptions import BuscoError
 import numpy as np
 
 logger = BuscoLogger.get_logger(__name__)
 
 
-class NoGenesError(SystemExit):
+class NoGenesError(BuscoError):
 
     def __init__(self):
         super().__init__("No genes were recognized by BUSCO. Please check the content of your input file.")
@@ -34,8 +34,9 @@ class AutoSelectLineage:
          "Because of overlapping markers/spurious matches among domains, busco matches in another domain do not "
          "necessarily mean that your genome/proteome contains sequences from this domain. "
          "However, a high busco score in multiple domains might help you identify possible contaminations.", logger)
-    def __init__(self, config):
-        self.config = config
+    def __init__(self, config_manager):
+        self.config_manager = config_manager
+        self.config = self.config_manager.config_main
         if self.config.getboolean("busco_run", "auto-lineage-prok"):
             self.all_lineages = ["archaea", "bacteria"]
         elif self.config.getboolean("busco_run", "auto-lineage-euk"):
@@ -71,6 +72,10 @@ class AutoSelectLineage:
         self.f_percents.append(f_percent)
         return
 
+    @classmethod
+    def reset(cls):
+        cls.runners = []
+
     @log("Running auto selector", logger, debug=True)
     def run_auto_selector(self):
         """
@@ -79,7 +84,7 @@ class AutoSelectLineage:
         """
         root_runners = self.run_lineages_list(self.all_lineages)
         self.get_best_match_lineage(root_runners)
-        if self.virus_check():
+        if self.virus_check() and not self.config.getboolean("busco_run", "auto-lineage-euk"):
             self.virus_pipeline = True
             self.run_virus_datasets()
             self.get_best_match_lineage(type(self).runners)
@@ -91,8 +96,6 @@ class AutoSelectLineage:
 
         logger.info("{} selected\n".format(os.path.basename(self.best_match_lineage_dataset)))
         self.config.set("busco_run", "domain_run_name", os.path.basename(self.best_match_lineage_dataset))
-        BuscoRunner.final_results.append(self.selected_runner.analysis.hmmer_runner.hmmer_results_lines)
-        BuscoRunner.results_datasets.append(os.path.basename(self.best_match_lineage_dataset))
         return
 
     def virus_check(self):
@@ -114,8 +117,8 @@ class AutoSelectLineage:
         root_runners = []
         for l in lineages_list:
             self.current_lineage = "{}_{}".format(l, self.dataset_version)
-            autoconfig = BuscoConfigAuto(self.config, self.current_lineage)
-            busco_run = BuscoRunner(autoconfig)
+            autoconfig = self.config_manager.load_busco_config_auto(self.current_lineage)
+            busco_run = AnalysisRunner(autoconfig)
             busco_run.run_analysis(callback=self.callback)
             root_runners.append(busco_run)
             type(self).runners.append(busco_run)  # Save all root runs so they can be recalled if chosen
@@ -132,7 +135,7 @@ class AutoSelectLineage:
         max_ind = inds[max_mask]
         return max_ind
 
-    def evaluate(self, runners):
+    def evaluate(self, runners, use_percent=False):
         """
         Evaluate output scores from all BUSCO runs. Lineage with the highest number of complete (single + multiple)
         copy BUSCOs is assigned as the best_match_lineage.
@@ -144,9 +147,9 @@ class AutoSelectLineage:
         """
         self.collate_results(runners)
 
-        max_ind = self.get_max_ind(np.array(self.s_buscos) + np.array(self.d_buscos))
+        max_ind = self.get_max_ind(np.array(self.s_percents) + np.array(self.d_percents)) if use_percent else self.get_max_ind(np.array(self.s_buscos) + np.array(self.d_buscos))
         if len(max_ind) > 1:
-            max_ind2 = self.get_max_ind(np.array(self.f_buscos)[max_ind])
+            max_ind2 = self.get_max_ind(np.array(self.f_percents)[max_ind]) if use_percent else self.get_max_ind(np.array(self.f_buscos)[max_ind])
             max_ind = max_ind[max_ind2]
             if len(max_ind) > 1:
                 if ((self.s_buscos[max_ind[0]] == 0.0)
@@ -167,10 +170,12 @@ class AutoSelectLineage:
         self.d_buscos = [runner.analysis.hmmer_runner.multi_copy for runner in runners]
         self.f_buscos = [runner.analysis.hmmer_runner.only_fragments for runner in runners]
         self.s_percents = [runner.analysis.hmmer_runner.s_percent for runner in runners]
+        self.d_percents = [runner.analysis.hmmer_runner.d_percent for runner in runners]
+        self.f_percents = [runner.analysis.hmmer_runner.f_percent for runner in runners]
         return
 
-    def get_best_match_lineage(self, runners):
-        max_ind = self.evaluate(runners)
+    def get_best_match_lineage(self, runners, use_percent=False):
+        max_ind = self.evaluate(runners, use_percent)
         self.selected_runner = runners[int(max_ind)]
         self.best_match_lineage_dataset = self.selected_runner.config.get("busco_run", "lineage_dataset")
         runners.pop(int(max_ind))
@@ -196,15 +201,15 @@ class AutoSelectLineage:
                 "Certain mollicute clades use a different genetic code to the rest of bacteria. They are not part "
                 "of the BUSCO placement tree and need to be tested separately. For more information, see the user "
                 "guide.")
-            self.check_mollicutes()
+            use_percent = self.selected_runner.mode == "proteins"
+            self.check_mollicutes(use_percent)
             if os.path.basename(self.selected_runner.config.get("busco_run", "lineage_dataset")).startswith("bacteria"):
                 logger.info("Bacteria domain is a better match than the mollicutes subclade. Continuing to tree placement.")
                 self.run_busco_placer()
             else:
                 logger.info("Mollicutes dataset is a better match for your data. Testing subclades...")
                 self._run_3_datasets(self.selected_runner)
-                BuscoRunner.final_results.append(self.selected_runner.analysis.hmmer_runner.hmmer_results_lines)
-                BuscoRunner.results_datasets.append(os.path.basename(self.best_match_lineage_dataset))
+
         elif ("geno" in self.selected_runner.mode
               and self.selected_runner.analysis.prodigal_runner.current_gc == "4"
               and os.path.basename(
@@ -212,18 +217,19 @@ class AutoSelectLineage:
             logger.info("The results from the Prodigal gene predictor indicate that your data belongs to the "
                         "mollicutes clade. Testing subclades...")
             self._run_3_datasets()
-            BuscoRunner.final_results.append(self.selected_runner.analysis.hmmer_runner.hmmer_results_lines)
-            BuscoRunner.results_datasets.append(os.path.basename(self.best_match_lineage_dataset))
         elif self.selected_runner.domain == "viruses":
             pass
         else:
             self.run_busco_placer()
         return
 
-    def check_mollicutes(self):
+    def set_best_match_lineage(self):
+        AnalysisRunner.selected_dataset = os.path.basename(self.best_match_lineage_dataset)
+
+    def check_mollicutes(self, use_percent=False):
         runners = self.run_lineages_list(["mollicutes"])
         runners.append(self.selected_runner)
-        self.get_best_match_lineage(runners)
+        self.get_best_match_lineage(runners, use_percent=use_percent)
         return
 
     def run_busco_placer(self):  # todo: revisit structure of this method after cleaning BuscoPlacer
@@ -252,9 +258,7 @@ class AutoSelectLineage:
         self.config.placement_files = placement_file_versions  # Necessary to pass these filenames to the final run to be recorded.
         lineage, supporting_markers, placed_markers = dataset_details
         lineage = "{}_{}".format(lineage, self.config.get("busco_run", "datasets_version"))  # todo: this should probably be done in buscoplacer
-        self.best_match_lineage_dataset = os.path.join(self.config.get("busco_run", "download_path"),
-                                                       "lineages",
-                                                       os.path.basename(lineage))
+        self.best_match_lineage_dataset = lineage  # basename
         return
 
     def _run_3_datasets(self, mollicutes_runner=None):
