@@ -19,9 +19,11 @@ import hashlib
 import urllib.request
 from urllib.error import URLError
 import gzip
+import pandas as pd
 
 from busco.BuscoLogger import BuscoLogger
 from busco.BuscoLogger import LogDecorator as log
+from busco.Exceptions import BatchFatalError, BuscoError
 
 logger = BuscoLogger.get_logger(__name__)
 
@@ -34,7 +36,7 @@ class BuscoDownloadManager:
     Else, a warning is produced.
     """
 
-    version_files = {}
+    version_files = None
 
     def __init__(self, config):
         """
@@ -46,7 +48,7 @@ class BuscoDownloadManager:
         self.download_base_url = config.get("busco_run", "download_base_url")
         self.local_download_path = config.get("busco_run", "download_path")
         self._create_main_download_dir()
-        if not type(self).version_files and not self.offline:
+        if not type(self).version_files is not None and not self.offline:
             self._load_versions()
 
     def _create_main_download_dir(self):
@@ -57,22 +59,26 @@ class BuscoDownloadManager:
     def _load_versions(self):
         try:
             versions_file = self._obtain_versions_file()
-            with open(versions_file, "r") as v_file:
-                for line in v_file:
-                    line = line.strip().split("\t")
-                    dataset_name = line[0]
-                    dataset_date = line[1]
-                    dataset_hash = line[2]
-                    type(self).version_files.update(
-                        {dataset_name: (dataset_date, dataset_hash)}
-                    )
+            type(self).version_files = pd.read_csv(
+                versions_file,
+                sep="\t",
+                names=["dataset", "date", "hash", "domain", "type"],
+                index_col="dataset",
+            )
         except URLError as e:
             if self.offline:
                 logger.warning(
                     "Unable to verify BUSCO datasets because of offline mode"
                 )
             else:
-                raise SystemExit(e)
+                raise BatchFatalError(e)
+        except FileNotFoundError:
+            logger.warning(
+                "Unable to find file_versions.tsv. This may be due to an internet access problem. "
+                "Attempting to continue in offline mode."
+            )
+            self.offline = True
+
         return
 
     @log("Downloading information on latest versions of BUSCO data...", logger)
@@ -82,7 +88,7 @@ class BuscoDownloadManager:
         try:
             urllib.request.urlretrieve(remote_filepath, local_filepath)
         except URLError:
-            SystemExit("Cannot reach {}".format(remote_filepath))
+            raise BatchFatalError("Cannot reach {}".format(remote_filepath))
         return local_filepath
 
     def _create_category_dir(self, category):
@@ -103,16 +109,16 @@ class BuscoDownloadManager:
                     dataset_date = line[1]
                     break
         if not dataset_date:
-            raise SystemExit(
+            raise BuscoError(
                 "Creation date could not be extracted from dataset.cfg file."
             )
         return dataset_date
 
     def _check_existing_version(self, local_filepath, category, data_basename):
         try:
-            latest_update = type(self).version_files[data_basename][0]
+            latest_update = type(self).version_files.loc[data_basename]["date"]
         except KeyError:
-            raise SystemExit(
+            raise BuscoError(
                 "{} is not a valid option for '{}'".format(data_basename, category)
             )
         path_basename, extension = os.path.splitext(data_basename)
@@ -145,7 +151,7 @@ class BuscoDownloadManager:
                 > 0
             )
 
-        hash = type(self).version_files[data_basename][1]
+        hash = type(self).version_files.loc[data_basename]["hash"]
 
         return present, up_to_date, latest_version, local_filepath, hash
 
@@ -154,7 +160,7 @@ class BuscoDownloadManager:
             logger.info("Using local {} directory {}".format(category, data_name))
             return data_name
         elif "/" in data_name:
-            raise SystemExit("{} does not exist".format(data_name))
+            raise BuscoError("{} does not exist".format(data_name))
         if self.offline:
             if category == "lineages":
                 local_dataset = os.path.join(
@@ -163,7 +169,7 @@ class BuscoDownloadManager:
                 if os.path.exists(local_dataset):
                     return local_dataset
                 else:
-                    raise SystemExit(
+                    raise BuscoError(
                         "Unable to run BUSCO in offline mode. Dataset {} does not "
                         "exist.".format(local_dataset)
                     )
@@ -180,9 +186,8 @@ class BuscoDownloadManager:
                 )
                 if len(placement_files) > 0:
                     return placement_files[-1]
-                    # todo: for offline mode, log which files are being used (in case of more than one glob match)
                 else:
-                    raise SystemExit(
+                    raise BuscoError(
                         "Unable to run BUSCO placer in offline mode. Cannot find necessary placement "
                         "files in {}".format(self.local_download_path)
                     )
@@ -263,7 +268,7 @@ class BuscoDownloadManager:
                 )
                 logger.info("deleting corrupted file {}".format(local_filepath))
                 os.remove(local_filepath)
-                raise SystemExit(
+                raise BuscoError(
                     "BUSCO was unable to download or update all necessary files"
                 )
             else:
@@ -290,14 +295,24 @@ class BuscoDownloadManager:
                 with open(unzipped_filename, "wb") as decompressed_file:
                     for line in compressed_file:
                         decompressed_file.write(line)
-            os.remove(local_filepath)
+            try:
+                os.remove(local_filepath)
+            except FileNotFoundError:
+                logger.warning(
+                    "Downloaded gzip file was removed before this BUSCO run could remove it."
+                )
             local_filepath = unzipped_filename
 
         if os.path.splitext(local_filepath)[1] == ".tar":
             untarred_filename = local_filepath.replace(".tar", "")
             with tarfile.open(local_filepath) as tar_file:
                 tar_file.extractall(os.path.dirname(local_filepath))
-            os.remove(local_filepath)
+            try:
+                os.remove(local_filepath)
+            except FileNotFoundError:
+                logger.warning(
+                    "Downloaded tarball was removed before this BUSCO run could remove it."
+                )
             local_filepath = untarred_filename
 
         return local_filepath
