@@ -31,6 +31,7 @@ import os
 import pandas as pd
 from collections import defaultdict
 import subprocess
+from busco.Exceptions import BuscoError
 
 logger = BuscoLogger.get_logger(__name__)
 
@@ -187,9 +188,12 @@ class GenomeAnalysisEukaryotesAugustus(BLASTAnalysis, GenomeAnalysisEukaryotes):
         super().__init__()
         self._long = self.config.getboolean("busco_run", "long")
         try:
-            self._target_species = self.config.get("busco_run", "augustus_species")
+            self._target_species_initial = self.config.get(
+                "busco_run", "augustus_species"
+            )
+            self._target_species = self._target_species_initial
         except KeyError:
-            raise SystemExit(
+            raise BuscoError(
                 "Something went wrong. Eukaryota datasets should specify an augustus species."
             )
         try:
@@ -218,8 +222,10 @@ class GenomeAnalysisEukaryotesAugustus(BLASTAnalysis, GenomeAnalysisEukaryotes):
 
     def cleanup(self):
         try:
-            if self._target_species.startswith("BUSCO"):
-                self.augustus_runner.move_retraining_parameters()
+            self.augustus_runner.move_retraining_parameters()
+            self.config.set(
+                "busco_run", "augustus_species", self._target_species_initial
+            )  # Reset parameter for batch mode
         except OSError:
             pass
         super().cleanup()
@@ -398,7 +404,7 @@ class GenomeAnalysisEukaryotesMetaeuk(GenomeAnalysisEukaryotes):
                 if i == 1:
                     logger.info("Metaeuk rerun did not find any genes")
                 else:
-                    raise SystemExit(
+                    raise BuscoError(
                         "Metaeuk did not find any genes in the input file."
                     )
 
@@ -406,7 +412,7 @@ class GenomeAnalysisEukaryotesMetaeuk(GenomeAnalysisEukaryotes):
             self.metaeuk_runner.combine_run_results()
         except FileNotFoundError:
             # This exception should only happen if the rerun file does not exist. If the initial run file was
-            # missing there would have been a SystemExit call above. The index 0 sets the "combined" file to the
+            # missing there would have been a BatchFatalError call above. The index 0 sets the "combined" file to the
             # output of the initial run.
             self.metaeuk_runner.combined_pred_protein_seqs = (
                 self.metaeuk_runner.pred_protein_mod_files[0]
@@ -430,11 +436,8 @@ class GenomeAnalysisEukaryotesMetaeuk(GenomeAnalysisEukaryotes):
     def validate_output(self):
         if len(self.metaeuk_runner.headers_files) < 2:
             return
-        hmmer_results = {
-            **self.hmmer_runner.is_complete,
-            **self.hmmer_runner.is_very_large,
-            **self.hmmer_runner.is_fragment,
-        }
+        hmmer_results = self.hmmer_runner.merge_dicts()
+
         if len(hmmer_results) > 0:
             exon_records = self.get_exon_records(hmmer_results)
             df = self.exons_to_df(exon_records)
@@ -535,6 +538,7 @@ class GenomeAnalysisEukaryotesMetaeuk(GenomeAnalysisEukaryotes):
                             strand,
                             score,
                             run_found,
+                            gene_id,
                         )
                         exon_records.append(record)
         return exon_records
@@ -548,10 +552,12 @@ class GenomeAnalysisEukaryotesMetaeuk(GenomeAnalysisEukaryotes):
                 busco_group = busco_groups.get_group(busco_id)
             except KeyError:  # if busco was removed during overlap filtering
                 continue
-            busco_score_groups = busco_group.groupby(["Score"])
-            for _, busco_score_group in busco_score_groups:
+            busco_gene_groups = busco_group.groupby("Orig gene ID")
+            for gene_match, busco_gene_group in busco_gene_groups:
+                if gene_match not in matches:
+                    continue
                 min_coord = None
-                for idx, row in busco_score_group.iterrows():
+                for idx, row in busco_gene_group.iterrows():
                     low_coord = row["Start"]
                     high_coord = row["Stop"]
                     score = row["Score"]
@@ -562,23 +568,28 @@ class GenomeAnalysisEukaryotesMetaeuk(GenomeAnalysisEukaryotes):
                     else:
                         min_coord = low_coord
                         max_coord = high_coord
-                for gene_match, details in matches.items():
-                    if details[0]["bitscore"] == score:
-                        new_gene_match = "{}:{}-{}".format(seq, min_coord, max_coord)
-                        hmmer_result_dict_new[busco_id].update(
-                            {new_gene_match: details}
-                        )
-                        matched_genes_new[new_gene_match].append(busco_id)
-                        self.gene_details[new_gene_match] = self.gene_details[
-                            gene_match
-                        ]
-                        self.sequences_aa[
-                            new_gene_match
-                        ] = self.metaeuk_runner.sequences_aa[gene_match]
+
+                details = matches[gene_match]
+                df_strand = busco_gene_group["Strand"].iloc[0]
+                new_gene_match = "{}:{}-{}".format(seq, min_coord, max_coord)
+                hmmer_result_dict_new[busco_id].update({new_gene_match: details})
+                matched_genes_new[new_gene_match].append(busco_id)
+                self.gene_details[new_gene_match] = [
+                    {
+                        "gene_start": min_coord,
+                        "gene_end": max_coord,
+                        "strand": df_strand,
+                    }
+                ]
+                self.sequences_aa[new_gene_match] = self.metaeuk_runner.sequences_aa[
+                    gene_match
+                ]
         return hmmer_result_dict_new, matched_genes_new
 
-    @log("Validating exons and removing overlapping matches", logger)
     def exons_to_df(self, records):
+        if self._mode == "genome":
+            logger.info("Validating exons and removing overlapping matches")
+
         df = self.metaeuk_runner.records_to_df(records)
         df["Start"] = df["Start"].astype(int)
         df["Stop"] = df["Stop"].astype(int)
@@ -744,7 +755,7 @@ class GenomeAnalysisEukaryotesMetaeuk(GenomeAnalysisEukaryotes):
             exon_matched = False
             exon_size_nt = int(entry["Stop"]) - int(entry["Start"]) + 1
             if not exon_size_nt % 3 == 0:
-                raise SystemExit(
+                raise BuscoError(
                     "The exon coordinates contain fractional reading frames and are ambiguous."
                 )
             exon_size_aa = exon_size_nt / 3
