@@ -4,6 +4,7 @@ from tests.unittests import ConfigManager_unittests
 from tests.unittests import BuscoConfig_unittests
 from tests.unittests import AutoLineage_unittests
 from tests.unittests import GenomeAnalysis_unittests
+from tests.unittests import BuscoRunner_unittests
 import sys
 
 loader = unittest.TestLoader()
@@ -14,6 +15,7 @@ suite.addTests(loader.loadTestsFromModule(ConfigManager_unittests))
 suite.addTests(loader.loadTestsFromModule(BuscoConfig_unittests))
 suite.addTests(loader.loadTestsFromModule(AutoLineage_unittests))
 suite.addTests(loader.loadTestsFromModule(GenomeAnalysis_unittests))
+suite.addTests(loader.loadTestsFromModule(BuscoRunner_unittests))
 
 runner = unittest.TextTestRunner(verbosity=3)
 result = runner.run(suite)
